#ifndef _PYGAME2_DOCBASE_H_
#define _PYGAME2_DOCBASE_H_

#define DOC_BASE "Base features for pygame2 that are shared shared and used throughout\nthe whole pygame2 package."
#define DOC_BASE_BUFFERPROXY "BufferProxy () -> BufferProxy\n\nCreates a new, empty BufferProxy.\n\nA buffer interface object that acts as a proxy on classes and\nattributes not featuring the buffer interface. A\nBufferProxy usually should be constructed from C code, not Python."
#define DOC_BASE_BUFFERPROXY_LENGTH "Gets the size of the buffer data in bytes."
#define DOC_BASE_BUFFERPROXY_RAW "Gets the raw buffer data as string. The string may contain\nNUL bytes."
#define DOC_BASE_BUFFERPROXY_WRITE "write (buffer, offset) -> None\n\nWrites raw data to the BufferProxy.\n\nWrites the raw data from *buffer* to the BufferProxy object,\nstarting at the specified *offset* within the BufferProxy. If\nthe length of the passed *buffer* exceeds the length of the\nBufferProxy (reduced by *offset*), an IndexError will\nbe raised."
#define DOC_BASE_COLOR "Color (...) -> Color\n\nCreates a new Color object.\n\nThe Color class represents RGBA color values using a value range of\n0-255. It allows basic arithmetic operations to create new colors,\nsupports conversions to other color spaces such as HSV or HSL and lets\nyou adjust single color channels. The following constructors are supported:\n\n* Color (0xAARRGGBB)\n* Color (0xRRGGBB)\n* Color (r, g, b, a)\n* Color (r, g, b)\n* Color (\"0xRRGGBBAA\")\n* Color (\"#RRGGBBAA\")\n* Color (\"name\")"
#define DOC_BASE_COLOR_A "Gets or sets the alpha value of the Color."
#define DOC_BASE_COLOR_B "Gets or sets the blue value of the Color."
#define DOC_BASE_COLOR_CMY "The CMY representation of the Color. The CMY\ncomponents are in the ranges C = [0, 1], M = [0, 1], Y = [0,\n1]. Note that this will not return the absolutely exact CMY values\nfor the set RGB values in all cases. Due to the RGB mapping from\n0-255 and the CMY mapping from 0-1 rounding errors may cause the\nCMY values to differ slightly from what you might expect."
#define DOC_BASE_COLOR_G "Gets or sets the green value of the Color."
#define DOC_BASE_COLOR_HSLA "The HSLA representation of the Color. The HSLA\ncomponents are in the ranges H = [0, 360], S = [0, 100], L = [0, 100],\nA = [0, 100]. Note that this will not return the absolutely exact HSL\nvalues for the set RGB values in all cases. Due to the RGB mapping from\n0-255 and the HSL mapping from 0-100 and 0-360 rounding errors may cause\nthe HSL values to differ slightly from what you might expect."
#define DOC_BASE_COLOR_HSVA "The HSVA representation of the Color. The HSVA\ncomponents are in the ranges H = [0, 360], S = [0, 100], V = [0, 100],\nA = [0, 100]. Note that this will not return the absolutely exact HSV\nvalues for the set RGB values in all cases. Due to the RGB mapping from\n0-255 and the HSV mapping from 0-100 and 0-360 rounding errors may cause\nthe HSV values todiffer slightly from what you might expect."
#define DOC_BASE_COLOR_I1I2I3 "The I1I2I3 representation of the Color.\nThe I1I2I3 components are in the ranges I1 = [0, 1], I2 = [-0.5, 0.5],\nI3 = [-0.5, 0.5]. Note that this will not return the absolutely exact\nI1I2I3 values for the set RGB values in all cases. Due to the RGB mapping\nfrom 0-255 and the I1I2I3 from 0-1 rounding errors may cause the I1I2I3\nvalues to differ slightly from what you might expect."
#define DOC_BASE_COLOR_R "Gets or sets the red value of the Color."
#define DOC_BASE_COLOR_CORRECT_GAMMA "correct_gamma (gamma) -> Color\n\nApplies a certain gamma value to the Color.\n\nApplies a certain *gamma* value to the Color and returns a new\nColor with the adjusted RGBA values. The *gamma* value will be\nused as exponent for a power of calculation to adjust the actual\ngamma tone.\n\nThe passed *gamma* value will influence he overall luminance of\nthe color, where the following rules apply.\n\n* Pure white and black colors (R = G = B = 255 or 0) won't be effected by adjusting the gamma value.\n* the smaller the gamma factor, the higher the luminance and thus the brighter the color channels will be."
#define DOC_BASE_COLOR_NORMALIZE "ormalize() -> (float, float, float, float)\n\nReturns the normalized RGBA values of the Color.\n\nReturns the normalized RGBA values of the Color as floating\npoint values."
#define DOC_BASE_ERROR "Error (...) -> Error\n\nA specialized exception class that indicates a misbehavior within the\npygame2 modules."
#define DOC_BASE_FRECT "FRect (...) -> FRect\n\nA class for storing rectangular coordinates.\n\nA FRect is used to store and manipulate rectangular coordinates."
#define DOC_BASE_FRECT_BOTTOM "Gets or sets the bottom edge position of the FRect."
#define DOC_BASE_FRECT_BOTTOMLEFT "Gets or sets the bottom left corner position of the FRect."
#define DOC_BASE_FRECT_BOTTOMRIGHT "Gets or sets the bottom right corner position of the FRect."
#define DOC_BASE_FRECT_CENTER "Gets or sets the center position of the FRect."
#define DOC_BASE_FRECT_CENTERX "Gets or sets the horizontal center position of the FRect."
#define DOC_BASE_FRECT_CENTERY "Gets or sets the vertical center position of the FRect."
#define DOC_BASE_FRECT_H "Gets or sets the height of the FRect."
#define DOC_BASE_FRECT_HEIGHT "Gets or sets the height of the FRect."
#define DOC_BASE_FRECT_LEFT "Gets or sets the left edge position of the FRect."
#define DOC_BASE_FRECT_MIDBOTTOM "Gets or sets the mid bottom edge position of the FRect."
#define DOC_BASE_FRECT_MIDLEFT "Gets or sets the mid left edge position of the FRect."
#define DOC_BASE_FRECT_MIDRIGHT "Gets or sets the mid right edge position of the FRect."
#define DOC_BASE_FRECT_MIDTOP "Gets or sets the mid top edge position of the FRect."
#define DOC_BASE_FRECT_RIGHT "Gets or sets the right position of the FRect."
#define DOC_BASE_FRECT_SIZE "Gets or sets the width and height of the FRect as\n2-value tuple."
#define DOC_BASE_FRECT_TOP "Gets or sets the top edge position of the FRect."
#define DOC_BASE_FRECT_TOPLEFT "Gets or sets the top left corner position of the FRect."
#define DOC_BASE_FRECT_TOPRIGHT "Gets or sets the top right corner position of the FRect."
#define DOC_BASE_FRECT_W "Gets or sets the width of the FRect."
#define DOC_BASE_FRECT_WIDTH "Gets or sets the width of the FRect."
#define DOC_BASE_FRECT_X "Gets or sets the horizontal top left position of the\nFRect."
#define DOC_BASE_FRECT_Y "Gets or sets the vertical top left position of the\nFRect."
#define DOC_BASE_FRECT_CEIL "ceil () -> Rect\n\nCreates a Rect from the specified FRect.\n\nThis creates a Rect using the smallest integral values greater\nor equal to the FRect floating point values."
#define DOC_BASE_FRECT_CLAMP "clamp (frect) -> FRect\n\nMoves the rectangle inside another.\n\nReturns a new rectangle that is moved to be completely inside\nthe argument FRect. If the rectangle is too large to\nfit inside, it is centered inside the argument FRect,\nbut its size is not changed."
#define DOC_BASE_FRECT_CLAMP_IP "clamp_ip (frect) -> None\n\nMoves the rectangle inside another, in place.\n\nSame as clamp, but operates in place."
#define DOC_BASE_FRECT_CLIP "clip (frect) -> FRect\n\nCrops a rectangle inside another.\n\nReturns a new rectangle that is cropped to be completely inside\nthe argument FRect. If the two rectangles do not\noverlap to begin with, a FRect with 0 size is\nreturned. Thus it returns the area, in which both rects overlap."
#define DOC_BASE_FRECT_COLLIDEDICT "collidedict (dict[, checkvals, key]) -> key, value\n\nTest if one rectangle in a dictionary intersects.\n\nReturns the key and value of the first dictionary entry that\ncollides with the FRect. If no collisions are found,\nNone is returned. Depending on the *checkvals* argument either the\nkeys or values of *dict* must be FRect objects. By default,\nthe keys are checked.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nFRect itself and a key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_FRECT_COLLIDEDICTALL "collidedictall (dict[, checkvals, key]) -> [(key, value), ...]\n\nTest if all rectangles in a dictionary intersect.\n\nReturns a list of all the key and value pairs that intersect\nwith the FRect. If no collisions are found an empty\nlist is returned. Depending on the *checkvals* argument either the\nkeys or values of the dict must be FRect objects. By\ndefault, the keys are checked.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nFRect itself and a key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_FRECT_COLLIDELIST "collidelist (rects[, key]) -> int\n\nTest if one rectangle in a list intersects.\n\nTest whether the rectangle collides with any in a sequence of\nrectangles. The index of the first collision found is\nreturned. If no collisions are found an index of -1 is returned.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nFRect itself and a key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_FRECT_COLLIDELISTALL "collidelistall (rects[, key]) -> [int, ...]\n\nTest if all rectangles in a list intersect.\n\nReturns a list of all the indices that contain rectangles that\ncollide with the FRect. If no intersecting rectangles\nare found, an empty list is returned.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nFRect itself and a key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_FRECT_COLLIDEPOINT "collidepoint (x, y) -> bool\ncollidepoint (point) -> bool\n\nTest if a point is inside a rectangle.\n\nReturns True if the given *point* is inside the rectangle. A point\nalong the right or bottom edge is not considered to be inside\nthe rectangle."
#define DOC_BASE_FRECT_COLLIDERECT "colliderect (frect) -> bool\n\nTest if two rectangles overlap.\n\nReturns True if any portion of either rectangle overlap (except\nthe top+bottom or left+right edges)."
#define DOC_BASE_FRECT_CONTAINS "contains (frect) -> bool\n\nTest if one rectangle is inside another.\n\nReturns True when the argument rectangle is completely inside\nthe FRect."
#define DOC_BASE_FRECT_COPY "copy () -> FRect\n\nCreates a copy of the FRect.\n\nReturns a new FRect, that contains the same values as the\ncaller."
#define DOC_BASE_FRECT_FIT "fit (frect) -> FRect\n\nResize and move a rectangle with aspect ratio.\n\nReturns a new rectangle that is moved and resized to fit\nanother. The aspect ratio of the original FRect is\npreserved, so the new rectangle may be smaller than the target\nin either width or height."
#define DOC_BASE_FRECT_FLOOR "floor () -> Rect\n\nCreates a Rect from the specified FRect.\n\nThis creates a Rect using the largest integral values less than\nor equal to the FRect floating point values."
#define DOC_BASE_FRECT_INFLATE "inflate (x, y) -> FRect\ninflate (size) -> FRect\n\nGrow or shrink the rectangle size.\n\nReturns a new rectangle with the size changed by the given\noffset. The rectangle remains centered around its current\ncenter. Negative values will shrink the rectangle."
#define DOC_BASE_FRECT_INFLATE_IP "inflate_ip (x, y) -> None\ninflate_ip (size) -> None\n\nGrow or shrink the rectangle size, in place.\n\nSame as inflate, but operates in place."
#define DOC_BASE_FRECT_MOVE "move (x, y) -> FRect\nmove (point) -> FRect\n\nMoves the rectangle.\n\nReturns a new rectangle that is moved by the given offset. The *x*\nand *y* arguments can be any float or integer value, positive or\nnegative."
#define DOC_BASE_FRECT_MOVE_IP "move_ip (x, y) -> None\nmove_ip (point) -> FRect\n\nMoves the rectangle, in place.\n\nSame as move, but operates in place."
#define DOC_BASE_FRECT_ROUND "round () -> :classRect\n\nCreates a Rect from the specified FRect.\n\nThis creates a Rect using the FRect floating\npoint values rounded to the nearest integral value."
#define DOC_BASE_FRECT_TRUNC "trunc () -> Rect\n\nCreates a Rect from the specified FRect.\n\nThis creates a Rect using truncated integral values from the\nFRect floating point values."
#define DOC_BASE_FRECT_UNION "union (frect) -> FRect\n\nJoins two rectangles into one.\n\nReturns a new rectangle that completely covers the area of the\ntwo provided rectangles. There may be area inside the new FRect\nthat is not covered by the originals."
#define DOC_BASE_FRECT_UNION_IP "union_ip (frect) -> FRect\n\nJoins two rectangles into one, in place.\n\nSame as union, but operates in place."
#define DOC_BASE_RECT "Rect (...) -> Rect\n\nA class for storing rectangular coordinates.\n\nA Rect is used to store and manipulate rectangular coordinates."
#define DOC_BASE_RECT_BOTTOM "Gets or sets the bottom edge position of the Rect."
#define DOC_BASE_RECT_BOTTOMLEFT "Gets or sets the bottom left corner position of the Rect."
#define DOC_BASE_RECT_BOTTOMRIGHT "Gets or sets the bottom right corner position of the Rect."
#define DOC_BASE_RECT_CENTER "Gets or sets the center position of the Rect."
#define DOC_BASE_RECT_CENTERX "Gets or sets the horizontal center position of the Rect."
#define DOC_BASE_RECT_CENTERY "Gets or sets the vertical center position of the Rect."
#define DOC_BASE_RECT_H "Gets or sets the height of the Rect."
#define DOC_BASE_RECT_HEIGHT "Gets or sets the height of the Rect."
#define DOC_BASE_RECT_LEFT "Gets or sets the left edge position of the Rect."
#define DOC_BASE_RECT_MIDBOTTOM "Gets or sets the mid bottom edge position of the Rect."
#define DOC_BASE_RECT_MIDLEFT "Gets or sets the mid left edge position of the Rect."
#define DOC_BASE_RECT_MIDRIGHT "Gets or sets the mid right edge position of the Rect."
#define DOC_BASE_RECT_MIDTOP "Gets or sets the mid top edge position of the Rect."
#define DOC_BASE_RECT_RIGHT "Gets or sets the right position of the Rect."
#define DOC_BASE_RECT_SIZE "Gets or sets the width and height of the Rect as\n2-value tuple."
#define DOC_BASE_RECT_TOP "Gets or sets the top edge position of the Rect."
#define DOC_BASE_RECT_TOPLEFT "Gets or sets the top left corner position of the Rect."
#define DOC_BASE_RECT_TOPRIGHT "Gets or sets the top right corner position of the Rect."
#define DOC_BASE_RECT_W "Gets or sets the width of the Rect."
#define DOC_BASE_RECT_WIDTH "Gets or sets the width of the Rect."
#define DOC_BASE_RECT_X "Gets or sets the horizontal top left position of the Rect."
#define DOC_BASE_RECT_Y "Gets or sets the vertical top left position of the Rect."
#define DOC_BASE_RECT_CLAMP "clamp (rect) -> Rect\n\nMoves the rectangle inside another.\n\nReturns a new rectangle that is moved to be completely inside\nthe argument Rect. If the rectangle is too large to fit inside,\nit is centered inside the argument Rect, but its size is not\nchanged."
#define DOC_BASE_RECT_CLAMP_IP "clamp_ip (rect) -> None\n\nMoves the rectangle inside another, in place.\n\nSame as clamp, but operates in place."
#define DOC_BASE_RECT_CLIP "clip (rect) -> Rect\n\nCrops a rectangle inside another.\n\nReturns a new rectangle that is cropped to be completely inside\nthe argument Rect. If the two rectangles do not overlap\nto begin with, a Rect with 0 size is returned. Thus it\nreturns the area, in which both rects overlap."
#define DOC_BASE_RECT_COLLIDEDICT "collidedict (dict[, checkvals, key]) -> key, value\n\nTest if one rectangle in a dictionary intersects.\n\nReturns the key and value of the first dictionary entry that\ncollides with the Rect. If no collisions are found,\nNone is returned. Depending on the *checkvals* argument either the\nkeys or values of the *dict* must be Rect objects. By\ndefault, the keys are checked.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nRect itself and a key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_RECT_COLLIDEDICTALL "collidedictall (dict[, checkvals, key]) -> [(key, value), ...]\n\nTest if all rectangles in a dictionary intersect.\n\nReturns a list of all the key and value pairs that intersect\nwith the Rect. If no collisions are found an empty list\nis returned. Depending on the *checkvals* argument either the keys\nor values of the *dict* must be Rect objects. By default,\nthe keys are checked.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nRect itself and an key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_RECT_COLLIDELIST "collidelist (list[, key]) -> int\n\nTest if one rectangle in a list intersects.\n\nTest whether the rectangle collides with any in a sequence of\nrectangles. The index of the first collision found is\nreturned. If no collisions are found an index of -1 is returned.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nRect itself and an key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_RECT_COLLIDELISTALL "collidelistall (list[, key]) -> [int, ...]\n\nTest if all rectangles in a list intersect.\n\nReturns a list of all the indices that contain rectangles that\ncollide with the Rect. If no intersecting rectangles are found,\nan empty list is returned.\n\nYou can provide your own comparision function as *key*\nargument. The comparision function will take two arguments, the\nRect itself and an key or value from the passed\ndictionary. It must return True or False \n\ndef cmpfunc (rect1, rect2):\n...\nreturn True"
#define DOC_BASE_RECT_COLLIDEPOINT "collidepoint (x, y) -> bool\ncollidepoint (point) -> bool\n\nTest if a point is inside a rectangle.\n\nReturns True if the given point is inside the rectangle. A point\nalong the right or bottom edge is not considered to be inside\nthe rectangle."
#define DOC_BASE_RECT_COLLIDERECT "colliderect (rect) -> bool\n\nTest if two rectangles overlap.\n\nReturns True if any portion of either rectangle overlap (except\nthe top+bottom or left+right edges)."
#define DOC_BASE_RECT_CONTAINS "contains (rect) -> bool\n\nTest if one rectangle is inside another.\n\nReturns True when the argument rectangle is completely inside\nthe Rect."
#define DOC_BASE_RECT_COPY "copy () -> Rect\n\nCreates a copy of the Rect.\n\nReturns a new Rect, that contains the same values as the\ncaller."
#define DOC_BASE_RECT_FIT "fit (rect) -> Rect\n\nResize and move a rectangle with aspect ratio.\n\nReturns a new rectangle that is moved and resized to fit\nanother. The aspect ratio of the original Rect is preserved, so\nthe new rectangle may be smaller than the target in either width\nor height."
#define DOC_BASE_RECT_INFLATE "inflate (x, y) -> Rect\ninflate (size) -> Rect\n\nGrow or shrink the rectangle size.\n\nReturns a new rectangle with the size changed by the given\noffset. The rectangle remains centered around its current\ncenter. Negative values will shrink the rectangle."
#define DOC_BASE_RECT_INFLATE_IP "inflate_ip (x, y) -> None\ninflate_ip (size) -> Rect\n\nGrow or shrink the rectangle size, in place.\n\nSame as inflate, but operates in place."
#define DOC_BASE_RECT_MOVE "move (x, y) -> Rect\nmove (point) -> Rect\n\nMoves the rectangle.\n\nReturns a new rectangle that is moved by the given offset. The *x*\nand *y* arguments can be any integer value, positive or negative."
#define DOC_BASE_RECT_MOVE_IP "move_ip (x, y) -> None\nmove_ip (point) -> None\n\nMoves the rectangle, in place.\n\nSame as move, but operates in place."
#define DOC_BASE_RECT_UNION "union (rect) -> Rect\n\nJoins two rectangles into one.\n\nReturns a new rectangle that completely covers the area of the\ntwo provided rectangles. There may be area inside the new Rect\nthat is not covered by the originals."
#define DOC_BASE_RECT_UNION_IP "union_ip (rect) -> Rect\n\nJoins two rectangles into one, in place.\n\nSame as union, but operates in place."
#define DOC_BASE_SURFACE "Surface () -> Surface\n\nAn abstract base class for arbitrary Surface objects.\n\nYou should not instantiate this class directly."
#define DOC_BASE_SURFACE_HEIGHT "Gets the height of the Surface."
#define DOC_BASE_SURFACE_PIXELS "Gets a buffer with the pixels of the Surface."
#define DOC_BASE_SURFACE_SIZE "Gets the width and height of the Surface."
#define DOC_BASE_SURFACE_WIDTH "Gets the width of the Surface."
#define DOC_BASE_SURFACE_BLIT "blit (**kwds) -> object\n\nPerforms a blit operation on the Surface.\n\nThe behavior, arguments and return value depend on the concrete\nSurface implementation."
#define DOC_BASE_SURFACE_COPY "copy () -> Surface\n\nCreates a copy of this Surface."
#define DOC_BASE_FONT "Font () -> Font\n\nAn abstract base class for arbitrary Font objects.\n\nYou should not instantiate this class directly."
#define DOC_BASE_FONT_HEIGHT "Gets the standard height of the Font typography."
#define DOC_BASE_FONT_SIZE "Gets the width and height of the Font typography."
#define DOC_BASE_FONT_NAME "Gets the name of the loaded Font."
#define DOC_BASE_FONT_STYLE "Gets or sets the style used to render the Font."
#define DOC_BASE_FONT_RENDER "render (**kwds) -> object\n\nRenders the specified text using the Font object. The exact input\narguments and return value is dependand on each specific Font\nimplementation."
#define DOC_BASE_FONT_COPY "copy () -> Font\n\nCreates a copy of this Font."

#endif
