#ifndef _PYGAME2_DOCMASK_H_
#define _PYGAME2_DOCMASK_H_

#define DOC_MASK "A module for image masks, suitable for fast pixel-perfect collision\ndetections."
#define DOC_MASK_FROM_SURFACE "from_surface (surface, threshold) -> Mask\n\nReturns a Mask from the given pygame2.sdl.video.Surface.\n\nMakes the transparent parts of the pygame2.sdl.video.Surface not\nset, and the opaque parts set. The alpha of each pixel is checked to see\nif it is greater than the given *threshold*. If the\npygame2.sdl.video.Surface is color keyed, then *threshold* is not\nused.\n\nThis requires pygame2 to be built with SDL support enabled."
#define DOC_MASK_FROM_THRESHOLD "from_threshold (surface, color[, threshold, thressurface]) -> Mask\n\nCreates a Mask by thresholding surfaces.\n\nThis is a more featureful method of getting a Mask from a\npygame2.sdl.video.Surface. If supplied with only one\npygame2.sdl.video.Surface, all pixels within the *threshold* of\nthe supplied *color* are set in the :classMask. If given\nthe optional *thressurface*, all pixels in *surface* that are within the\n*threshold* of the corresponding pixel in *thressurface* are set in the\nMask."
#define DOC_MASK_MASK "Mask (width, height) -> Mask\nMask (size) -> Mask\n\nCreates a new Mask object with the specified width and height."
#define DOC_MASK_MASK_ANGLE "Gets the orientation of the pixels. Finds the approximate\norientation of the pixels in the image from -90 to 90\ndegrees. This works best if performed on one connected component\nof pixels. It will return 0.0 on an empty Mask."
#define DOC_MASK_MASK_CENTROID "Gets the centroid, the center of pixel mass, of the pixels\nin a Mask. Returns a coordinate tuple for the centroid\nof the Mask. If the Mask is empty, it will return (0, 0)."
#define DOC_MASK_MASK_COUNT "Gets the amount of set bits in the Mask."
#define DOC_MASK_MASK_HEIGHT "Gets the height of the Mask."
#define DOC_MASK_MASK_SIZE "Gets the width and height of the Mask as tuple."
#define DOC_MASK_MASK_WIDTH "Gets the width of the Mask."
#define DOC_MASK_MASK_CLEAR "clear () -> None\n\nClears all bits in the Mask.\n\nResets the state of all bits in the Mask to 0."
#define DOC_MASK_MASK_CONNECTED_COMPONENT "connected_component ([x, y]) -> Mask\nconnected_component ([point]) -> Mask\n\nReturns a Mask of a connected region of pixels.\n\nThis uses the SAUF algorithm to find a connected component in\nthe Mask. It checks 8 point connectivity. By default,\nit will return the largest connected component in the\nimage. Optionally, a coordinate pair of a pixel can be\nspecified, and the connected component containing it will be\nreturned. In the event the pixel at that location is not set,\nthe returned Mask will be empty. The Mask\nreturned is the same size as the original Mask."
#define DOC_MASK_MASK_CONNECTED_COMPONENTS "connected_components ([min]) -> Mask\n\nReturns a Mask of a connected region of pixels.\n\nReturns a list of masks of connected regions of pixels. An\noptional minimum number of pixels per connected region can be\nspecified to filter out noise."
#define DOC_MASK_MASK_CONVOLVE "convolve (mask[, outputmask, point]) -> Mask\nconvolve (mask[, outputmask, x, y]) -> Mask\n\nReturn the convolution with another Mask.\n\nReturns a Mask with the [x-offset[0], y-offset[1]] bit\nset if shifting *mask* so that it's lower right corner pixel is\nat (*x*, *y*) would cause it to overlap with self.\n\nIf an *outputmask* is specified, the output is drawn onto\n*outputmask* and *outputmask* is returned. Otherwise a mask of\nsize size + size - (1, 1) is created."
#define DOC_MASK_MASK_DRAW "draw (mask, x, y) -> None\ndraw (mask, point) -> None\n\nDraws the passed Mask onto the Mask.\n\nThis performs a bitwise OR operation upon the calling\nMask. The passed *mask*'s start offset for the draw\noperation will be the *x* and *y* offset passed to the method."
#define DOC_MASK_MASK_ERASE "erase (mask, x, y) -> None\nerase (mask, point) -> None\n\nErases the passed Mask from the Mask.\n\nThis performs a bitwise NAND operation upon the calling\nMask. The passed *mask*'s start offset for the erase\noperation will be the *x* and *y* offset passed to the method."
#define DOC_MASK_MASK_FILL "fill () -> None\n\nSets all bits to 1 within the Mask."
#define DOC_MASK_MASK_GET_AT "get_at (x, y) -> int\nget_at (point) -> int\n\nGets the bit value at the desired location."
#define DOC_MASK_MASK_GET_BOUNDING_RECTS "get_bounding_rects () -> [Mask, Mask, ...]\n\nReturns a list of bounding rects of regions of set pixels.\n\nThis gets a bounding rect of connected regions of set bits. A bounding\nrect is one for which each of the connected pixels is inside the rect."
#define DOC_MASK_MASK_INVERT "invert () -> None\n\nInverts all bits in the Mask."
#define DOC_MASK_MASK_OUTLINE "outline ([skip]) -> [(x1, y1), (x2, y2), ...]\n\nGets the points outlining an object on the mask.\n\nReturns a list of points of the outline of the first object it\ncomes across in the Mask. For this to be useful, there\nshould probably only be one connected component of pixels in the\nMask. The *skip* option allows you to skip pixels in\nthe outline. For example, setting it to 10 would return a list\nof every 10th pixel in the outline."
#define DOC_MASK_MASK_OVERLAP "overlap (mask, x, y) -> int, int\noverlap (mask, point) -> int, int\n\nReturns nonzero if the masks overlap with the given offset.\n\nThe overlap tests uses the following offsets (which may be negative): \n\n+----+----------...\n|A   | yoffset\n|  +-+----------...\n+--|B\n|xoffset\n|  |\n|  |\n:  :"
#define DOC_MASK_MASK_OVERLAP_AREA "overlap_area (mask, x, y) -> int\noverlap_area (mask, point) -> int\n\nReturns the number of overlapping bits of two Masks.\n\nThis returns how many pixels overlap with the other *mask* given. It\ncan be used to see in which direction things collide, or to see how\nmuch the two masks collide."
#define DOC_MASK_MASK_OVERLAP_MASK "overlap_mask (mask, x, y) -> Mask\noverlap_mask (mask, point) -> Mask\n\nReturns a Mask with the overlap of two other masks. A bitwise\nAND."
#define DOC_MASK_MASK_SCALE "scale (width, height) -> Mask\nscale (size) -> Mask\n\nCreates a new scaled Mask with the given *width* and *height*.\n\nThe quality of the scaling may not be perfect for all circumstances,\nbut it should be reasonable. If either *width* or *height* is 0 a clear\n1x1 mask is returned."
#define DOC_MASK_MASK_SET_AT "set_at (x, y) -> None\nset_at (point) -> None\n\nSets the bit value at the desired location."

#endif
