#ifndef _PYGAME2_DOCBASE_H_
#define _PYGAME2_DOCBASE_H_

#define DOC_BASE "Basic extensions for the pygame2 SDL wrapper."
#define DOC_BASE_PIXELARRAY "PixelArray (surface) -> PixelArray\n\nCreates a new PixelArray from the specified\npygame2.sdl.video.Surface.\n\nThe PixelArray wraps up a pygame2.sdl.video.Surface and provides\na direct 2D array access to its pixels using the surface's columns as\nfirst and its rows as second axis. It supports slicing and row and pixel\nmanipluation while in place operations such as addition, subtraction,\nmultiplication, division and so forth are not allowed.\n\nWhile it is possible to assign both, integer color values and RGB(A)\ncolor tuples, the PixelArray will only use integers for the\ncolor representation. Thus, checking for certain colors has to be done\nusing the pygame2.sdl.video.Surface.map_rgb method of the surface,\nthe PixelArray was created for. \n\npxarray = pygame.PixelArray (surface)\n# Check, if the first pixel at the topleft corner is blue\nif pxarray[0][0] == surface.map_rgb ((0, 0, 255)):\n...\n\nIf you operate on a slice, you also can use arbitrary sequences or\nother PixelArray objects to modify the pixels. They have to match\nthe size of the PixelArray however. \n\npxarray[a:b] = 0xFF00FF                   # set all pixels to 0xFF00FF\npxarray[a:b] = (0xFF00FF, 0xAACCEE, ... ) # first pixel = 0xFF00FF,\n# second pixel  = 0xAACCEE, ...\npxarray[a:b] = ((255, 0, 255), (170, 204, 238), ...) # same as above\npxarray[a:b] = ((255, 0, 255), 0xAACCEE, ...)        # same as above\npxarray[a:b] = otherarray[x:y]            # slice sizes must match\n\nNote, that something like \n\npxarray[2:4][3:5] = ...\n\nwill not cause a rectangular manipulation. Instead it will be\nfirst sliced to a two-column array, which then shall be sliced by\ncolumns once more, which will fail due an IndexError. This is\ncaused by the slicing mechanisms in python and an absolutely\ncorrect behavior. Create a single columned slice first, which you\ncan manipulate then: \n\npxarray[2][3:5] = ...\npxarray[3][3:5] = ...\n\nIf you wan" \
"t to make a rectangular manipulation or create a view of\na part of the PixelArray, you also can use the subscript\nabilities. You can easily create different views by creating\n'subarrays' using the subscripts. \n\n# Create some new PixelArray objects providing a different view\n# of the original array/surface.\nnewarray = pxarray[2:4,3:5]\notherarray = pxarray[2,2]\n\nSubscripts also can be used to do fast rectangular pixel\nmanipulations instead of iterating over the x or y axis as above. \n\npxarray[2,:] = (0, 0, 0)                # Make each second column black.\n\nDuring its lifetime, the PixelArray locks the surface, thus you\nexplicitly have to delete it once it is not used anymore and the\nsurface should perform operations in the same scope."
#define DOC_BASE_PIXELARRAY_SURFACE "Gets the pygame2.sdl.video.Surface the PixelArray uses."
#define DOC_BASE_PIXELARRAY_COMPARE "compare (array, distance=0, weights=(0.299, 0.587, 0.114)) -> PixelArray\n\nCompares the PixelArray with another one.\n\nCompares the contents of the PixelArray with those from the\npassed PixelArray. It returns a new PixelArray with a\nblack/white color mask that indicates the differences (white) of\nboth arrays. Both PixelArray objects must have indentical bit\ndepths and dimensions.\n\nIt uses a simple weighted euclidian distance formula to\ncalculate the distance between the colors. The *distance* space\nranges from 0.0 to 1.0 and is used as threshold for the color\ndetection. This causes the comparision to mark pixels with a\nsimilar, but not exactly identical color, as black."
#define DOC_BASE_PIXELARRAY_EXTRACT "extract (color, distance=0, weights=(0.299, 0.587, 0.114)) -> PixelArray\n\nExtracts the passed color from the PixelArray.\n\nExtracts the passed *color* by changing all matching pixels to\nwhite, while non-matching pixels are changed to black. This\nreturns a new PixelArray with the black/white color mask.\n\nIt uses a simple weighted euclidian distance formula to calculate\nthe distance between the colors. The *distance* space ranges from\n0.0 to 1.0 and is used as threshold for the color detection.\nThis causes the extraction to take pixels with a similar, but\nnot exactly identical color, into account as well."
#define DOC_BASE_PIXELARRAY_MAKE_SURFACE "make_surface () -> pygame2.sdl.video.Surface\n\nCreates a new pygame2.sdl.video.Surface from the current\nPixelArray.\n\nCreates a new pygame2.sdl.video.Surface from the current\nPixelArray. Depending on the current PixelArray the size, pixel order\netc. will be different from the original\npygame2.sdl.video.Surface. \n\n# Create a new surface flipped around the vertical axis.\nsf = pxarray[:,-1].make_surface ()"
#define DOC_BASE_PIXELARRAY_REPLACE "replace (color, repcolor, distance=0, weights=(0.299, 0.587, 0.114)) -> None\n\nReplaces the passed color in the PixelArray with another one.\n\nReplaces the pixels with the passed *color* in the PixelArray\nby changing them them to the passed replacement color.\n\nIt uses a simple weighted euclidian distance formula to calculate\nthe distance between the colors. The *distance* space ranges from 0.0\nto 1.0 and is used as threshold for the color detection. This\ncauses the replacement to take pixels with a similar, but not\nexactly identical color, into account as well.\n\nThis is an in place operation that directly affects the pixels\nof the PixelArray."

#endif
