#ifndef _PYGAME2_DOCNUMERICSURFARRAY_H_
#define _PYGAME2_DOCNUMERICSURFARRAY_H_

#define DOC_NUMERICSURFARRAY ".. note\n\nThis module is not supported under Python 3.x.\n\n.. note\n\nYou should favour the usage :mod:pygame2.sdlext.surfarray over\ndirectly using the numpy or Numeric surfarray modules, if you do\nnot require any special features of one of the packages. This\nallows you to write compatible code for various platforms having\neither Numeric *or* numpy installed.\n\nFunctions to convert pixel data between pygame Surfaces and Numeric\narrays. This module will only be available when pygame is built with\nsupport for the external Numeric package.\n\nNote, that numericsurfarray is an optional module. It requires that\nNumeric is installed to be used. If not installed, an exception will\nbe raised when it is used. eg. ImportError: no module named Numeric\n\nEvery pixel is stored as a single integer value to represent the\nred, green, and blue colors. The 8bit images use a value that looks\ninto a colormap. Pixels with higher depth use a bit packing process\nto place three or four values into a single number.\n\nThe Numeric arrays are indexed by the X axis first, followed by the Y\naxis. Arrays that treat the pixels as a single integer are referred\nto as 2D arrays. This module can also separate the red, green, and\nblue color values into separate indices. These types of arrays are\nreferred to as 3D arrays, and the last index is 0 for red, 1 for\ngreen, and 2 for blue."
#define DOC_NUMERICSURFARRAY_ARRAY2D "array2d (Surface) -> array\n\nCopy pixels into a 2d array.\n\nCopy the pixels from a Surface into a 2D array. The bit depth of the\nsurface will control the size of the integer values, and will work\nfor any type of pixel format.\n\nThis function will temporarily lock the Surface as pixels are copied\n(see the Surface.lock - lock the Surface memory for pixel access\nmethod)."
#define DOC_NUMERICSURFARRAY_ARRAY3D "array3d (Surface) -> array\n\nCopy pixels into a 3d array.\n\nCopy the pixels from a Surface into a 3D array. The bit depth of the\nsurface will control the size of the integer values, and will work\nfor any type of pixel format.\n\nThis function will temporarily lock the Surface as pixels are copied\n(see the Surface.lock - lock the Surface memory for pixel access\nmethod)."
#define DOC_NUMERICSURFARRAY_ARRAY_ALPHA "array_alpha (Surface) -> array\n\nCopy pixel alphas into a 2d array.\n\nCopy the pixel alpha values (degree of transparency) from a\nSurface into a 2D array. This will work for any type of Surface\nformat. Surfaces without a pixel alpha will return an array with\nall opaque values.\n\nThis function will temporarily lock the Surface as pixels are\ncopied (see the Surface.lock - lock the Surface memory for pixel\naccess method)."
#define DOC_NUMERICSURFARRAY_ARRAY_COLORKEY "array_colorkey (Surface) -> array\n\nCopy the colorkey values into a 2d array.\n\nCreate a new array with the colorkey transparency value from each\npixel. If the pixel matches the colorkey it will be fully\ntranparent; otherwise it will be fully opaque.\n\nThis will work on any type of Surface format. If the image has no\ncolorkey a solid opaque array will be returned.\n\nThis function will temporarily lock the Surface as pixels are\ncopied."
#define DOC_NUMERICSURFARRAY_BLIT_ARRAY "blit_array (Surface, array) -> None\n\nBlit directly from a array values\n\nDirectly copy values from an array into a Surface. This is faster\nthan converting the array into a Surface and blitting. The array\nmust be the same dimensions as the Surface and will completely\nreplace all pixel values.\n\nThis function will temporarily lock the Surface as the new values\nare copied."
#define DOC_NUMERICSURFARRAY_MAKE_SURFACE "make_surface (array) -> Surface\n\nCopy an array to a new surface.\n\nCreate a new Surface that best resembles the data and format on the\narray. The array can be 2D or 3D with any sized integer values."
#define DOC_NUMERICSURFARRAY_MAP_ARRAY "map_array (Surface, array3d) -> array2d\n\nMap a 3d array into a 2d array.\n\nConvert a 3D array into a 2D array. This will use the given\nSurface format to control the conversion. Palette surface formats\nare not supported.\n\nNote: arrays do not need to be 3D, as long as the minor axis has\nthree elements giving the component colours, any array shape can be\nused (for example, a single colour can be mapped, or an array of\ncolours)."
#define DOC_NUMERICSURFARRAY_PIXELS2D "pixels2d (Surface) -> array\n\nReference pixels into a 2d array.\n\nCreate a new 2D array that directly references the pixel values in a\nSurface. Any changes to the array will affect the pixels in the\nSurface. This is a fast operation since no data is copied.\n\nPixels from a 24-bit Surface cannot be referenced, but all other\nSurface bit depths can.\n\nThe Surface this references will remain locked for the lifetime of\nthe array (see the Surface.lock - lock the Surface memory for pixel\naccess method)."
#define DOC_NUMERICSURFARRAY_PIXELS3D "pixels3d (Surface) -> array\n\nReference pixels into a 3d array.\n\nCreate a new 3D array that directly references the pixel values in a\nSurface. Any changes to the array will affect the pixels in the\nSurface. This is a fast operation since no data is copied.\n\nThis will only work on Surfaces that have 24-bit or 32-bit\nformats. Lower pixel formats cannot be referenced.\n\nThe Surface this references will remain locked for the lifetime of\nthe array (see the Surface.lock - lock the Surface memory for pixel\naccess method)."
#define DOC_NUMERICSURFARRAY_PIXELS_ALPHA "pixels_alpha (Surface) -> array\n\nReference pixel alphas into a 2d array.\n\nCreate a new 2D array that directly references the alpha values\n(degree of transparency) in a Surface. Any changes to the array will\naffect the pixels in the Surface. This is a fast operation since no\ndata is copied.\n\nThis can only work on 32-bit Surfaces with a per-pixel alpha value.\n\nThe Surface this array references will remain locked for the\nlifetime of the array."

#endif
