#ifndef _PYGAME2_DOCTRANSFORM_H_
#define _PYGAME2_DOCTRANSFORM_H_

#define DOC_TRANSFORM "SDL surface transformation algorithms\n\nA surface transform is an operation that moves or resizes the pixels. All\nthese functions take a surface to operate on and return a new surface with\nthe results.\n\nSome of the transforms are considered destructive. This means every time\nthey are performed they lose pixel data. Common examples of this are\nresizing and rotating. For this reason, it is better to retransform the\noriginal surface than to keep transforming an image multiple times.\n(For example, suppose you are animating a bouncing spring which expands and\ncontracts. If you applied the size changes incrementally to the previous\nimages, you would lose detail. Instead, always begin with the original\nimage and scale to the desired size.)"
#define DOC_TRANSFORM_AVERAGE_COLOR "average_color (surface[, rect]) -> pygame2.Color\n\nFinds the average color of a surface.\n\nFinds the average color of a pygame2.sdl.video.Surface and\nreturns it. The optional *rect* argument can be used to find the average\ncolor for a specific region on the pygame2.sdl.video.Surface."
#define DOC_TRANSFORM_AVERAGE_SURFACES "average_surfaces ((surface1, surface2, ...)[, destsurface]) -> pygame2.sdl.video.Surface\n\nCreates a new surface with the average colors from a list of surfaces.\n\nThis finds the average color for each of the passed surfaces and creates\na new pygame2.sdl.video.Surface with those colors. If a\ndestination surface is passed, it will be used instead of creating a new\npygame2.sdl.video.Surface.\n\n.. note\n\nThe destination surface must have the same size as the largest\npygame2.sdl.video.Surface of the passed list."
#define DOC_TRANSFORM_CHOP "chop (surface, rect) -> pygame2.sdl.video.Surface\n\nExtracts a portion of a surface.\n\nExtracts a portion of a *surface*. All vertical and horizontal pixels\nsurrounding the given rectangle area are removed. The corner areas\n(diagonal to the *rect*) are then brought together and the result will be\nreturned as a new pygame2.sdl.video.Surface\n\n.. note\n\nIf you want a \"crop\" that returns the part of an image within a\nrect, you can blit with a rect to a new surface."
#define DOC_TRANSFORM_FLIP "flip (surface, flipx, flipy) -> pygame2.sdl.video.Surface\n\nFlips a surface horizontically, vertically or over both axes.\n\nThis can flip a *surface* either vertically, horizontally, or both.\nFlipping a Surface is nondestructive and returns a new\npygame2.sdl.video.Surface with the same dimensions."
#define DOC_TRANSFORM_GET_FILTERTYPE "get_filtertype () -> int\n\nGets the currently set filter type."
#define DOC_TRANSFORM_SET_FILTERTYPE "set_filtertype (type) -> int\n\nTries to apply a certain filter mechanism to use.\n\nTries to apply a certain filter mechanism for the :func:smoothscale\nfunction. Depending on which architecture and how Pygame2 was built,\nnot all available filters from :mod:pygame2.sdlext.constants might\nbe supported. In case the passed *type* is not supported for the\nexecuting system, the filter type will be automatically set back to\nFILTER_C.\n\nThe return value will be the effective filter type used."
#define DOC_TRANSFORM_LAPLACIAN "laplacian (surface[, destsurface]) -> pygame2.sdl.video.Surface\n\nFinds the edges in a surface using the laplacian algorithm.\n\nFinds the edges in a *surface* using the laplacian algorithm. If a\ndestination surface is provided, it will be used for drawing the results\ninstead of creating a new pygame2.sdl.video.Surface.\n\n.. note\n\nThe destination surface must have the same size as the *surface* to\nfind the edges in."
#define DOC_TRANSFORM_ROTATE "rotate (surface, angle) -> pygame2.sdl.video.Surface\n\nRotates a surface.\n\nRotates a *surface* using the specified *angle* in degrees. For positive\nangles, the rotation will be counterclockwise, negative angles will cause\na clockwise rotation.\n\nUnless rotating by 90 degree increments, the *surface* will be padded\nlarger to hold the new size. If the image has pixel alphas, the padded\narea will be transparent. Otherwise a color will picked that matches the\nsurface colorkey or the topleft pixel value."
#define DOC_TRANSFORM_SCALE "scale (surface, width, height[, destsurface]) -> pygame2.sdl.video.Surface\nscale (surface, size[, destsurface]) -> pygame2.sdl.video.Surface\n\nResizes the surface to a new resolution.\n\nThis is a fast scale operation that does not sample the results. An\noptional destination surface can be used, rather than have it create a\nnew one. This is faster if you want to repeatedly scale something.\n\n.. note\n\nThe destination surface must cover the size of the scaled result\nand feature the same format."
#define DOC_TRANSFORM_SCALE2X "scale (surface[, destsurface]) -> pygame2.sdl.video.Surface\n\nScales a surface to the double of its original size.\n\nThis will return a new pygame2.sdl.video.Surface that is double\nthe size of the original. It uses the AdvanceMAME Scale2X algorithm which\ndoes a 'jaggie-less' scale of bitmap graphics.\n\nThis really only has an effect on simple images with solid colors. On\nphotographic and antialiased images it will look like a regular\nunfiltered scale. An optional destination surface can be used, rather\nthan to have it create a new one. This is faster if you want to\nrepeatedly scale something.\n\n.. note\n\nThe destination surface must be twice the size of the *surface*\npassed in and feature the same format."
#define DOC_TRANSFORM_SMOOTHSCALE "smoothscale (surface, width, height[, destsurface]) -> pygame2.sdl.video.Surface\nsmoothscale (surface, size[, destsurface]) -> pygame2.sdl.video.Surface\n\nResizes the surface to a new resolution using smoothing algorithms.\n\nUses one of two different algorithms for scaling each dimension of the\ninput surface as required. For shrinkage, the output pixels are area\naverages of the colors they cover. For expansion, a bilinear filter is\nused.\n\nFor AMD64 and i686 architectures, optimized MMX and SSE filter routines\nare included and will run much faster than other machine types. The filter\nroutines can be get and set using :func:get_filtertype and\n:func:set_filtertype\n\nThis function only works for 24-bit or 32-bit surfaces. An exception will\nbe thrown if the input surface bit depth is less than 24.\n\n.. note\n\nThe destination surface must cover the size of the scaled result\nand feature the same format."
#define DOC_TRANSFORM_THRESHOLD "threshold (surface, color, threscolor[, destsurface]) -> pygame2.sdl.video.Surface\nthreshold (surface, colorsurface, threscolor[, destsurface]) -> pygame2.sdl.video.Surface\n\nTODO"

#endif
